
/*******************************
 * Types
 *******************************/

export type Balance = {
    currency: string,
    amount: number,
    // exchange / margin
    kind: string,
    quote: string,
    quote_equivalent: number,
}

export type CurrencyPair = {
    base: string,
    quote: string
}

export type EventProp = {
    id: number,
    name: string,
    tick: number
}

export type FirstConnectMessage = {
    ticks: Array<number>,
    prices: Array<number>,
    positions: Array<PositionProp>,
    balances: Array<Balance>
}

export type GetProfitLossMessage = {
    start: number,
    end: number
}

export type NewEventMessage = {
    tick: number,
    kind: string,
}

export type NewTickMessage = {
    tick: number,
    price: number,
    positions: Array<PositionProp>,
    balances: Array<Balance>
}

export type PositionCloseMessage = {
    message_name: string,
    position_id: number,
}

export type PositionProp = {
    id: number,
    state: string,
    base_price: number,
    amount: number,
    pair: CurrencyPair,
    profit_loss: number,
    profit_loss_percentage: number
}

export type PutProfitLossMessage = {
    pl: number,
    pl_perc: number
}

/*******************************
* ENUMS
*******************************/

export enum EventName {
    NewTick = "new_tick",
    FirstConnect = "first_connect",
    NewEvent = "new_event",
    ClosePosition = "close_position",
    GetProfitLoss = "get_profit_loss",
    PutProfitLoss = "put_profit_loss"
}

